@@ -6,28 +6,35 @@ import re
 import shutil
 import subprocess
 import sys
+import uuid
+import tarfile
 import typing
 from collections import OrderedDict
 from datetime import date
 from functools import lru_cache
+from pathlib import Path
+from typing import Optional
 from urllib.parse import urlparse
 
 # imports - third party imports
 import click
 import git
+import semantic_version as sv
 
 # imports - module imports
 import bench
-from bench.exceptions import NotInBenchDirectoryError
 from bench.utils import (
 	UNSET_ARG,
 	fetch_details_from_tag,
+	get_app_cache_extract_filter,
 	get_available_folder_name,
+	get_bench_cache_path,
 	is_bench_directory,
 	is_git_url,
 	is_valid_frappe_branch,
 	log,
 	run_frappe_cmd,
+	get_file_md5,
 )
 from bench.utils.bench import build_assets, install_python_dev_dependencies
 from bench.utils.render import step
@@ -40,21 +47,21 @@ logger = logging.getLogger(bench.PROJECT_NAME)
 
 
 class AppMeta:
-	def __init__(self, name: str, branch: str = None, to_clone: bool = True):
+	def __init__(self, name: str, branch: str | None = None, to_clone: bool = True):
 		"""
 		name (str): This could look something like
-		        1. https://github.com/frappe/healthcare.git
-		        2. git@github.com:frappe/healthcare.git
-		        3. frappe/healthcare@develop
-		        4. healthcare
-		        5. healthcare@develop, healthcare@v13.12.1
+				1. https://github.com/frappe/healthcare.git
+				2. git@github.com:frappe/healthcare.git
+				3. frappe/healthcare@develop
+				4. healthcare
+				5. healthcare@develop, healthcare@v13.12.1
 
 		References for Version Identifiers:
 		 * https://www.python.org/dev/peps/pep-0440/#version-specifiers
 		 * https://docs.npmjs.com/about-semantic-versioning
 
 		class Healthcare(AppConfig):
-		        dependencies = [{"frappe/erpnext": "~13.17.0"}]
+				dependencies = [{"frappe/erpnext": "~13.17.0"}]
 		"""
 		self.name = name.rstrip("/")
 		self.remote_server = "github.com"
@@ -95,7 +102,7 @@ class AppMeta:
 		# fetch meta for repo from remote git server - traditional get-app url
 		elif is_git_url(self.name):
 			self.is_url = True
-			self._setup_details_from_git_url()
+			self.__setup_details_from_git()
 
 		# fetch meta from new styled name tags & first party apps on github
 		else:
@@ -110,7 +117,7 @@ class AppMeta:
 		# If app is a git repo
 		self.git_repo = git.Repo(self.mount_path)
 		try:
-			self._setup_details_from_git_url(self.git_repo.remotes[0].url)
+			self.__setup_details_from_git(self.git_repo.remotes[0].url)
 			if not (self.branch or self.tag):
 				self.tag = self.branch = self.git_repo.active_branch.name
 		except IndexError:
@@ -121,12 +128,10 @@ class AppMeta:
 			self.tag = self.branch = None
 
 	def _setup_details_from_name_tag(self):
-		self.org, self.repo, self.tag = fetch_details_from_tag(self.name)
+		using_cached = bool(self.cache_key)
+		self.org, self.repo, self.tag = fetch_details_from_tag(self.name, using_cached)
 		self.tag = self.tag or self.branch
 
-	def _setup_details_from_git_url(self, url=None):
-		return self.__setup_details_from_git(url)
-
 	def __setup_details_from_git(self, url=None):
 		name = url if url else self.name
 		if name.startswith("git@") or name.startswith("ssh://"):
@@ -163,9 +168,10 @@ class App(AppMeta):
 	def __init__(
 		self,
 		name: str,
-		branch: str = None,
-		bench: "Bench" = None,
+		branch: str | None = None,
+		bench: "Bench | None" = None,
 		soft_link: bool = False,
+		cache_key=None,
 		*args,
 		**kwargs,
 	):
@@ -173,6 +179,8 @@ class App(AppMeta):
 		self.soft_link = soft_link
 		self.required_by = None
 		self.local_resolution = []
+		self.cache_key = cache_key
+		self.pyproject = None
 		super().__init__(name, branch, *args, **kwargs)
 
 	@step(title="Fetching App {repo}", success="App {repo} Fetched")
@@ -227,10 +235,13 @@ class App(AppMeta):
 		resolved=False,
 		restart_bench=True,
 		ignore_resolution=False,
+		using_cached=False,
 	):
 		import bench.cli
 		from bench.utils.app import get_app_name
 
+		self.validate_app_dependencies()
+
 		verbose = bench.cli.verbose or verbose
 		app_name = get_app_name(self.bench.name, self.app_name)
 		if not resolved and self.app_name != "frappe" and not ignore_resolution:
@@ -247,6 +258,7 @@ class App(AppMeta):
 			skip_assets=skip_assets,
 			restart_bench=restart_bench,
 			resolution=self.local_resolution,
+			using_cached=using_cached,
 		)
 
 	@step(title="Cloning and installing {repo}", success="App {repo} Installed")
@@ -256,7 +268,10 @@ class App(AppMeta):
 
 	@step(title="Uninstalling App {repo}", success="App {repo} Uninstalled")
 	def uninstall(self):
-		self.bench.run(f"{self.bench.python} -m pip uninstall -y {self.name}")
+		if os.environ.get("BENCH_USE_UV"):
+			self.bench.run(f"uv pip uninstall {self.name} --python {self.bench.python}")
+		else:
+			self.bench.run(f"{self.bench.python} -m pip uninstall -y {self.name}")
 
 	def _get_dependencies(self):
 		from bench.utils.app import get_required_deps, required_apps_from_hooks
@@ -284,6 +299,310 @@ class App(AppMeta):
 			required=self.local_resolution,
 		)
 
+	def get_pyproject(self) -> Optional[dict]:
+		from bench.utils.app import get_pyproject
+
+		if self.pyproject:
+			return self.pyproject
+
+		apps_path = os.path.join(os.path.abspath(self.bench.name), "apps")
+		pyproject_path = os.path.join(apps_path, self.app_name, "pyproject.toml")
+		self.pyproject = get_pyproject(pyproject_path)
+		return self.pyproject
+
+	def validate_app_dependencies(self, throw=False) -> None:
+		pyproject = self.get_pyproject() or {}
+		deps: Optional[dict] = (
+			pyproject.get("tool", {}).get("bench", {}).get("frappe-dependencies")
+		)
+		if not deps:
+			return
+
+		for dep, version in deps.items():
+			validate_dependency(self, dep, version, throw=throw)
+
+	"""
+	Get App Cache
+
+	Since get-app affects only the `apps`, `env`, and `sites`
+	bench sub directories. If we assume deterministic builds
+	when get-app is called, the `apps/app_name` sub dir can be
+	cached.
+
+	In subsequent builds this would save time by not having to:
+	- clone repository
+	- install frontend dependencies
+	- building frontend assets
+	as all of this is contained in the `apps/app_name` sub dir.
+
+	Code that updates the `env` and `sites` subdirs still need
+	to be run.
+	"""
+
+	def get_app_path(self) -> Path:
+		return Path(self.bench.name) / "apps" / self.app_name
+
+	def get_app_cache_temp_path(self, is_compressed=False) -> Path:
+		cache_path = get_bench_cache_path("apps")
+		ext = "tgz" if is_compressed else "tar"
+		tarfile_name = f"{self.app_name}.{uuid.uuid4().hex}.{ext}"
+		return cache_path / tarfile_name
+
+	def get_app_cache_hashed_path(self, temp_path: Path) -> Path:
+		assert self.cache_key is not None
+
+		ext = temp_path.suffix[1:]
+		md5 = get_file_md5(temp_path)
+		tarfile_name = f"{self.app_name}.{self.cache_key}.md5-{md5}.{ext}"
+
+		return temp_path.with_name(tarfile_name)
+
+	def get_cached(self) -> bool:
+		if not self.cache_key:
+			return False
+
+		if not (cache_path := validate_cache_and_get_path(self.app_name, self.cache_key)):
+			return False
+
+		app_path = self.get_app_path()
+		if app_path.is_dir():
+			shutil.rmtree(app_path)
+
+		click.secho(
+			f"Bench app-cache: extracting {self.app_name} from {cache_path.as_posix()}",
+		)
+
+		mode = "r:gz" if cache_path.suffix.endswith(".tgz") else "r"
+		with tarfile.open(cache_path, mode) as tar:
+			extraction_filter = get_app_cache_extract_filter(count_threshold=150_000)
+			try:
+				tar.extractall(app_path.parent, filter=extraction_filter)
+				click.secho(
+					f"Bench app-cache: extraction succeeded for {self.app_name}",
+					fg="green",
+				)
+			except Exception:
+				message = f"Bench app-cache: extraction failed for {self.app_name}"
+				click.secho(
+					message,
+					fg="yellow",
+				)
+				logger.exception(message)
+				shutil.rmtree(app_path)
+				return False
+
+		return True
+
+	def set_cache(self, compress_artifacts=False) -> bool:
+		if not self.cache_key:
+			return False
+
+		app_path = self.get_app_path()
+		if not app_path.is_dir():
+			return False
+
+		cwd = os.getcwd()
+		cache_path = self.get_app_cache_temp_path(compress_artifacts)
+		mode = "w:gz" if compress_artifacts else "w"
+
+		message = f"Bench app-cache: caching {self.app_name}"
+		if compress_artifacts:
+			message += " (compressed)"
+		click.secho(message)
+
+		self.prune_app_directory()
+
+		success = False
+		os.chdir(app_path.parent)
+		try:
+			with tarfile.open(cache_path, mode) as tar:
+				tar.add(app_path.name)
+
+			hashed_path = self.get_app_cache_hashed_path(cache_path)
+			unlink_no_throw(hashed_path)
+
+			cache_path.rename(hashed_path)
+			click.secho(
+				f"Bench app-cache: caching succeeded for {self.app_name} as {hashed_path.as_posix()}",
+				fg="green",
+			)
+
+			success = True
+		except Exception as exc:
+			log(f"Bench app-cache: caching failed for {self.app_name} {exc}", level=3)
+			success = False
+		finally:
+			os.chdir(cwd)
+		return success
+
+	def prune_app_directory(self):
+		app_path = self.get_app_path()
+		if can_frappe_use_cached(self):
+			remove_unused_node_modules(app_path)
+
+
+def coerce_url_to_name_if_possible(git_url: str, cache_key: str) -> str:
+	app_name = os.path.basename(git_url)
+	if can_get_cached(app_name, cache_key):
+		return app_name
+	return git_url
+
+
+def can_get_cached(app_name: str, cache_key: str) -> bool:
+	"""
+	Used before App is initialized if passed `git_url` is a
+	file URL as opposed to the app name.
+
+	If True then `git_url` can be coerced into the `app_name` and
+	checking local remote and fetching can be skipped while keeping
+	get-app command params the same.
+	"""
+
+	if cache_path := get_app_cache_path(app_name, cache_key):
+		return cache_path.exists()
+
+	return False
+
+
+def can_frappe_use_cached(app: App) -> bool:
+	min_frappe = get_required_frappe_version(app)
+	if not min_frappe:
+		return False
+
+	try:
+		return sv.Version(min_frappe) in sv.SimpleSpec(">=15.12.0")
+	except ValueError:
+		# Passed value is not a version string, it's an expression
+		pass
+
+	try:
+		"""
+		15.12.0 is the first version to support USING_CACHED,
+		but there is no way to check the last version without
+		support. So it's not possible to have a ">" filter.
+
+		Hence this excludes the first supported version.
+		"""
+		return sv.Version("15.12.0") not in sv.SimpleSpec(min_frappe)
+	except ValueError:
+		click.secho(
+			f"Bench app-cache: invalid value found for frappe version '{min_frappe}'",
+			fg="yellow",
+		)
+		# Invalid expression
+		return False
+
+
+def validate_dependency(app: App, dep: str, req_version: str, throw=False) -> None:
+	dep_path = Path(app.bench.name) / "apps" / dep
+	if not dep_path.is_dir():
+		click.secho(f"Required frappe-dependency '{dep}' not found.", fg="yellow")
+		if throw:
+			sys.exit(1)
+		return
+
+	dep_version = get_dep_version(dep, dep_path)
+	if not dep_version:
+		return
+
+	if sv.Version(dep_version) not in sv.SimpleSpec(req_version):
+		click.secho(
+			f"Installed frappe-dependency '{dep}' version '{dep_version}' "
+			f"does not satisfy required version '{req_version}'. "
+			f"App '{app.name}' might not work as expected.",
+			fg="yellow",
+		)
+		if throw:
+			click.secho(f"Please install '{dep}{req_version}' first and retry", fg="red")
+			sys.exit(1)
+
+
+def get_dep_version(dep: str, dep_path: Path) -> Optional[str]:
+	from bench.utils.app import get_pyproject
+
+	dep_pp = get_pyproject(str(dep_path / "pyproject.toml"))
+	version = dep_pp.get("project", {}).get("version")
+	if version:
+		return version
+
+	dinit_path = dep_path / dep / "__init__.py"
+	if not dinit_path.is_file():
+		return None
+
+	with dinit_path.open("r", encoding="utf-8") as dinit:
+		for line in dinit:
+			if not line.startswith("__version__ =") and not line.startswith("VERSION ="):
+				continue
+
+			version = line.split("=")[1].strip().strip("\"'")
+			if version:
+				return version
+			else:
+				break
+
+	return None
+
+
+def get_required_frappe_version(app: App) -> Optional[str]:
+	pyproject = app.get_pyproject() or {}
+
+	# Reference: https://github.com/frappe/bench/issues/1524
+	req_frappe = (
+		pyproject.get("tool", {})
+		.get("bench", {})
+		.get("frappe-dependencies", {})
+		.get("frappe")
+	)
+
+	if not req_frappe:
+		click.secho(
+			"Required frappe version not set in pyproject.toml, "
+			"please refer: https://github.com/frappe/bench/issues/1524",
+			fg="yellow",
+		)
+
+	return req_frappe
+
+
+def remove_unused_node_modules(app_path: Path) -> None:
+	"""
+	Erring a bit the side of caution; since there is no explicit way
+	to check if node_modules are utilized, this function checks if Vite
+	is being used to build the frontend code.
+
+	Since most popular Frappe apps use Vite to build their frontends,
+	this method should suffice.
+
+	Note: root package.json is ignored cause those usually belong to
+	apps that do not have a build step and so their node_modules are
+	utilized during runtime.
+	"""
+
+	for p in app_path.iterdir():
+		if not p.is_dir():
+			continue
+
+		package_json = p / "package.json"
+		if not package_json.is_file():
+			continue
+
+		node_modules = p / "node_modules"
+		if not node_modules.is_dir():
+			continue
+
+		can_delete = False
+		with package_json.open("r", encoding="utf-8") as f:
+			package_json = json.loads(f.read())
+			build_script = package_json.get("scripts", {}).get("build", "")
+			can_delete = "vite build" in build_script
+
+		if can_delete:
+			click.secho(
+				f"Bench app-cache: removing {node_modules.as_posix()}",
+				fg="yellow",
+			)
+			shutil.rmtree(node_modules)
+
 
 def make_resolution_plan(app: App, bench: "Bench"):
 	"""
@@ -346,6 +665,8 @@ def get_app(
 	soft_link=False,
 	init_bench=False,
 	resolve_deps=False,
+	cache_key=None,
+	compress_artifacts=False,
 ):
 	"""bench get-app clones a Frappe App from remote (GitHub or any other git server),
 	and installs it on the current bench. This also resolves dependencies based on the
@@ -359,8 +680,18 @@ def get_app(
 	from bench.bench import Bench
 	from bench.utils.app import check_existing_dir
 
+	use_cached_app = False
+
 	bench = Bench(bench_path)
-	app = App(git_url, branch=branch, bench=bench, soft_link=soft_link)
+	app = App(
+		git_url, branch=branch, bench=bench, soft_link=soft_link, cache_key=cache_key
+	)
+
+	if urlparse(git_url).scheme == "file" and cache_key:
+		if use_cached_app := app.get_cached():
+			git_url = coerce_url_to_name_if_possible(git_url, cache_key)
+			app = App(git_url, branch=branch, bench=bench, soft_link=soft_link, cache_key=cache_key)
+
 	git_url = app.url
 	repo_name = app.repo
 	branch = app.tag
@@ -382,10 +713,12 @@ def get_app(
 
 	if not is_bench_directory(bench_path):
 		if not init_bench:
-			raise NotInBenchDirectoryError(
+			click.secho(
 				f"{os.path.realpath(bench_path)} is not a valid bench directory. "
-				"Run with --init-bench if you'd like to create a Bench too."
+				"Run with --init-bench if you'd like to create a Bench too.",
+				fg="red",
 			)
+			sys.exit(1)
 
 		from bench.utils.system import init
 
@@ -418,6 +751,15 @@ def get_app(
 		)
 		return
 
+	if use_cached_app:
+		app.install(
+			verbose=verbose,
+			skip_assets=skip_assets,
+			restart_bench=restart_bench,
+			using_cached=True,
+		)
+		return
+
 	dir_already_exists, cloned_path = check_existing_dir(bench_path, repo_name)
 	to_clone = not dir_already_exists
 
@@ -443,6 +785,8 @@ def get_app(
 	):
 		app.install(verbose=verbose, skip_assets=skip_assets, restart_bench=restart_bench)
 
+	app.set_cache(compress_artifacts)
+
 
 def install_resolved_deps(
 	bench,
@@ -516,9 +860,11 @@ def install_resolved_deps(
 
 def new_app(app, no_git=None, bench_path="."):
 	if bench.FRAPPE_VERSION in (0, None):
-		raise NotInBenchDirectoryError(
-			f"{os.path.realpath(bench_path)} is not a valid bench directory."
+		click.secho(
+			f"{os.path.realpath(bench_path)} is not a valid bench directory.",
+			fg="red",
 		)
+		sys.exit(1)
 
 	# For backwards compatibility
 	app = app.lower().replace(" ", "_").replace("-", "_")
@@ -550,9 +896,11 @@ def install_app(
 	restart_bench=True,
 	skip_assets=False,
 	resolution=UNSET_ARG,
+	using_cached=False,
 ):
 	import bench.cli as bench_cli
 	from bench.bench import Bench
+	from bench.utils.system import get_mariadb_pkgconfig_path, check_pkg_config
 
 	install_text = f"Installing {app}"
 	click.secho(install_text, fg="yellow")
@@ -570,21 +918,53 @@ def install_app(
 
 	app_path = os.path.realpath(os.path.join(bench_path, "apps", app))
 
-	bench.run(
-		f"{bench.python} -m pip install {quiet_flag} --upgrade -e {app_path} {cache_flag}"
-	)
+	env = None
+
+	# macOS needs a custom PKG_CONFIG_DIR for frappe v16+
+	from bench.utils.app import get_current_frappe_version
+
+	if app == "frappe" and get_current_frappe_version(bench_path) >= 16:
+		check_pkg_config()
+
+		if sys.platform == "darwin":
+			env = {
+				"PKG_CONFIG_PATH": get_mariadb_pkgconfig_path(),
+			}
+
+	if os.environ.get("BENCH_USE_UV"):
+		try:
+			bench.run(
+				f"uv pip install {quiet_flag} --upgrade -e {app_path} {cache_flag} --python {bench.python}",
+				env=env,
+			)
+		except Exception as e:
+			print(f"Error occured during app install: {e}", file=sys.stderr)
+			sys.exit(167)
+	else:
+		try:
+			bench.run(
+				f"{bench.python} -m pip install {quiet_flag} --upgrade -e {app_path} {cache_flag}",
+				env=env,
+			)
+		except Exception as e:
+			print(f"Error occured during app install: {e}", file=sys.stderr)
+			sys.exit(167)
 
 	if conf.get("developer_mode"):
-		install_python_dev_dependencies(apps=app, bench_path=bench_path, verbose=verbose)
+		install_python_dev_dependencies(
+			apps=app, bench_path=bench_path, verbose=verbose
+		)
 
-	if os.path.exists(os.path.join(app_path, "package.json")):
-		yarn_install = "yarn install --verbose" if verbose else "yarn install"
+	if not using_cached and os.path.exists(os.path.join(app_path, "package.json")):
+		yarn_install = "yarn install --check-files"
+		if verbose:
+			yarn_install += " --verbose"
 		bench.run(yarn_install, cwd=app_path)
 
 	bench.apps.sync(app_name=app, required=resolution, branch=tag, app_dir=app_path)
 
 	if not skip_assets:
-		build_assets(bench_path=bench_path, app=app)
+		build_assets(bench_path=bench_path, app=app, using_cached=using_cached)
 
 	if restart_bench:
 		# Avoiding exceptions here as production might not be set-up
@@ -621,9 +1001,9 @@ Cannot proceed with update: You have local changes in app "{app}" that are not c
 Here are your choices:
 
 1. Merge the {app} app manually with "git pull" / "git pull --rebase" and fix conflicts.
-1. Temporarily remove your changes with "git stash" or discard them completely
+2. Temporarily remove your changes with "git stash" or discard them completely
 	with "bench update --reset" or for individual repositries "git reset --hard"
-2. If your changes are helpful for others, send in a pull request via GitHub and
+3. If your changes are helpful for others, send in a pull request via GitHub and
 	wait for them to be merged in the core."""
 					)
 					sys.exit(1)
@@ -706,3 +1086,58 @@ def get_apps_json(path):
 
 	with open(path) as f:
 		return json.load(f)
+
+
+def is_cache_hash_valid(cache_path: Path) -> bool:
+	parts = cache_path.name.split(".")
+	if len(parts) < 2 or not parts[-2].startswith("md5-"):
+		return False
+
+	md5 = parts[-2].split("-")[1]
+	return get_file_md5(cache_path) == md5
+
+
+def unlink_no_throw(path: Path):
+	if not path.exists():
+		return
+
+	try:
+		path.unlink(True)
+	except Exception:
+		pass
+
+
+def get_app_cache_path(app_name: str, cache_key: str) -> "Optional[Path]":
+	cache_path = get_bench_cache_path("apps")
+	glob_pattern = f"{app_name}.{cache_key}.md5-*"
+
+	for app_cache_path in cache_path.glob(glob_pattern):
+		return app_cache_path
+
+	return None
+
+
+def validate_cache_and_get_path(app_name: str, cache_key: str) -> "Optional[Path]":
+	if not cache_key:
+		return
+
+	if not (cache_path := get_app_cache_path(app_name, cache_key)):
+		return
+
+	if not cache_path.is_file():
+		click.secho(
+			f"Bench app-cache: file check failed for {cache_path.as_posix()}, skipping cache",
+			fg="yellow",
+		)
+		unlink_no_throw(cache_path)
+		return
+
+	if not is_cache_hash_valid(cache_path):
+		click.secho(
+			f"Bench app-cache: hash validation failed for {cache_path.as_posix()}, skipping cache",
+			fg="yellow",
+		)
+		unlink_no_throw(cache_path)
+		return
+
+	return cache_path
