@@ -5,10 +5,13 @@ import os
 import re
 import subprocess
 import sys
+import hashlib
 from functools import lru_cache
 from glob import glob
+from pathlib import Path
 from shlex import split
-from typing import List, Tuple
+from tarfile import TarInfo
+from typing import List, Optional, Tuple
 
 # imports - third party imports
 import click
@@ -21,6 +24,12 @@ from bench.exceptions import (
 	InvalidRemoteException,
 )
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+	from typing import Optional
+
+
 logger = logging.getLogger(PROJECT_NAME)
 paths_in_app = ("hooks.py", "modules.txt", "patches.txt")
 paths_in_bench = ("apps", "sites", "config", "logs", "config/pids")
@@ -34,6 +43,9 @@ def is_bench_directory(directory=os.path.curdir):
 	for folder in paths_in_bench:
 		path = os.path.abspath(os.path.join(directory, folder))
 		is_bench = is_bench and os.path.exists(path)
+		# Once is_bench becomes false, it will always be false, even if other path exists.
+		if not is_bench:
+			break
 
 	return is_bench
 
@@ -51,6 +63,17 @@ def is_frappe_app(directory: str) -> bool:
 	return bool(is_frappe_app)
 
 
+def get_bench_cache_path(sub_dir: Optional[str]) -> Path:
+	relative_path = "~/.cache/bench"
+	if sub_dir and not sub_dir.startswith("/"):
+		relative_path += f"/{sub_dir}"
+
+	cache_path = os.path.expanduser(relative_path)
+	cache_path = Path(cache_path)
+	cache_path.mkdir(parents=True, exist_ok=True)
+	return cache_path
+
+
 @lru_cache(maxsize=None)
 def is_valid_frappe_branch(frappe_path: str, frappe_branch: str):
 	"""Check if a branch exists in a repo. Throws InvalidRemoteException if branch is not found
@@ -109,6 +132,9 @@ def check_latest_version():
 	if VERSION.endswith("dev"):
 		return
 
+	if os.environ.get("FRAPPE_DOCKER_BUILD"):
+		return
+
 	import requests
 	from semantic_version import Version
 
@@ -406,7 +432,7 @@ def get_env_frappe_commands(bench_path=".") -> List:
 	return []
 
 
-def find_org(org_repo):
+def find_org(org_repo, using_cached: bool = False):
 	import requests
 
 	org_repo = org_repo[0]
@@ -418,10 +444,17 @@ def find_org(org_repo):
 		if res.ok:
 			return org, org_repo
 
-	raise InvalidRemoteException(f"{org_repo} not found in frappe or erpnext")
+	if using_cached:
+		return "", org_repo
+
+	raise InvalidRemoteException(
+		f"{org_repo} not found under frappe or erpnext GitHub accounts"
+	)
 
 
-def fetch_details_from_tag(_tag: str) -> Tuple[str, str, str]:
+def fetch_details_from_tag(
+	_tag: str, using_cached: bool = False
+) -> Tuple[str, str, str]:
 	if not _tag:
 		raise Exception("Tag is not provided")
 
@@ -436,7 +469,7 @@ def fetch_details_from_tag(_tag: str) -> Tuple[str, str, str]:
 	try:
 		org, repo = org_repo
 	except Exception:
-		org, repo = find_org(org_repo)
+		org, repo = find_org(org_repo, using_cached)
 
 	return org, repo, tag
 
@@ -559,3 +592,53 @@ def get_cmd_from_sysargv():
 		break
 
 	return cmd_from_ctx
+
+
+def get_app_cache_extract_filter(
+	count_threshold: int = 10_000,
+	size_threshold: int = 1_000_000_000,
+):  # -> Callable[[TarInfo, str], TarInfo | None]
+	state = dict(count=0, size=0)
+
+	AbsoluteLinkError = Exception
+
+	def data_filter(m: TarInfo, _: str) -> TarInfo:
+		return m
+
+	if (
+		sys.version_info.major == 3 and sys.version_info.minor > 7
+	) or sys.version_info.major > 3:
+		from tarfile import data_filter, AbsoluteLinkError
+
+	def filter_function(member: TarInfo, dest_path: str) -> Optional[TarInfo]:
+		state["count"] += 1
+		state["size"] += member.size
+
+		if state["count"] > count_threshold:
+			raise RuntimeError(f"Number of entries exceeds threshold ({state['count']})")
+
+		if state["size"] > size_threshold:
+			raise RuntimeError(f"Extracted size exceeds threshold ({state['size']})")
+
+		try:
+			return data_filter(member, dest_path)
+		except AbsoluteLinkError:
+			# Links created by `frappe` after extraction
+			return None
+
+	return filter_function
+
+
+def get_file_md5(p: Path) -> "str":
+	with open(p.as_posix(), "rb") as f:
+		try:
+			file_md5 = hashlib.md5(usedforsecurity=False)
+
+		# Will throw if < 3.9, can be removed once support
+		# is dropped
+		except TypeError:
+			file_md5 = hashlib.md5()
+
+		while chunk := f.read(2**16):
+			file_md5.update(chunk)
+	return file_md5.hexdigest()
